package com.yihu.hos.common.scheduler;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.yihu.hos.common.constants.MonitorConstant;
import com.yihu.hos.common.util.MongodbUtil;
import com.yihu.hos.common.util.SigarUtil;
import com.yihu.hos.core.datatype.DateUtil;
import com.yihu.hos.core.log.Logger;
import com.yihu.hos.core.log.LoggerFactory;
import com.yihu.hos.services.ServiceMonitorService;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import org.bson.Document;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 *  服务器性能数据采集定时器
 * @author HZY
 * @vsrsion 1.0
 * Created at 2016/10/11.
 */
@Component
public class MonitorScheduler {

    static private final Logger logger = LoggerFactory.getLogger(MonitorScheduler.class);
    private static String host = SigarUtil.getHost();
    @Resource(name = ServiceMonitorService.BEAN_ID)
    private ServiceMonitorService serviceMonitorService;

    @Scheduled(cron="0 0/1 * * * ?") //每分钟执行一次
    public void statusCheck() {
        System.out.println("每分钟执行一次。开始============================================");
        //TODO 采集服务器健康监控指标数据 statusTask.healthCheck();
//        collectEnvHealth();
        try {
            collectServiceHealth();
        } catch (JsonProcessingException e) {
            e.printStackTrace();
        }
        System.out.println("每分钟执行一次。结束。");
    }

    public String collectEnvHealth(){
        try {
            MongodbUtil monoEnv = new MongodbUtil("envHealth");
            Document result = null;
            result = new Document();
            result.put("create_date", DateUtil.getCurrentString(DateUtil.DEFAULT_YMDHMSDATE_FORMAT));
            result.put("create_time", new Date());

            //cpu
            JSONObject cpu = JSONObject.fromObject( SigarUtil.cpu());
            result.put("data",cpu);
            result.put("type", MonitorConstant.CPU);
            monoEnv.insert(host,result);
            //内存
            JSONObject memory = JSONObject.fromObject( SigarUtil.memory());
            result.put("data",memory);
            result.put("type", MonitorConstant.MEMORY);
            result.remove("_id");
            monoEnv.insert(host,result);
            //硬盘
            List<JSONObject> files = JSONArray.fromObject( SigarUtil.file());
            result.put("data",files);
            result.put("type", MonitorConstant.FILES);
            result.remove("_id");
            monoEnv.insert(host, result);
            //网络
            JSONObject net = JSONObject.fromObject( SigarUtil.net());
            result.put("data",net);
            result.put("type", MonitorConstant.NET);
            result.remove("_id");
            monoEnv.insert(host,result);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return "";
    }

    public void collectServiceHealth() throws JsonProcessingException {
        Date now = new Date();
        Date beforeDate = new Date(now.getTime() - 60000);
        String beginTime = DateUtil.toString(beforeDate, DateUtil.DEFAULT_YMDHMSDATE_FORMAT);
        String endTime = DateUtil.toString(now, DateUtil.DEFAULT_YMDHMSDATE_FORMAT);
        serviceMonitorService.bandwidth(beginTime, endTime);
        serviceMonitorService.qps(beginTime, endTime);
        serviceMonitorService.delay(beginTime, endTime);
        serviceMonitorService.usage(beginTime, endTime);
    }

//    @Scheduled(fixedRate=10000)
//    public void testTasks() {
//        logger.info("每10秒执行一次。开始……");
//        //statusTask.healthCheck();
//        logger.info("每10秒执行一次。结束。");
//    }

}
